/*****************************************************************************
 * Copyright (c) 2019, Nations Technologies Inc.
 *
 * All rights reserved.
 * ****************************************************************************
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer below.
 *
 * Nations' name may not be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * DISCLAIMER: THIS SOFTWARE IS PROVIDED BY NATIONS "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL NATIONS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ****************************************************************************/
 
/**
*\*\file bsp_led.h
*\*\author Nations
*\*\version v1.0.0
*\*\copyright Copyright (c) 2019, Nations Technologies Inc. All rights reserved.
**/


#ifndef __BSP_LED_H__
#define __BSP_LED_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "n32g430.h"

/** Define the GPIO port to which the LED is connected **/
#define LED_GPIO_PORT    	GPIOB			            /* GPIO port */
#define LED_GPIO_CLK 	    RCC_AHB_PERIPH_GPIOB		/* GPIO port clock */
#define LED_GPIO_PIN		GPIO_PIN_0			        /* GPIO connected to the SCL clock line */


/** Define macros that control IO **/
#define LED_TOGGLE		    {LED_GPIO_PORT->POD ^= LED_GPIO_PIN;}
#define LED_ON		        {LED_GPIO_PORT->PBSC = LED_GPIO_PIN;}
#define LED_OFF			      {LED_GPIO_PORT->PBC = LED_GPIO_PIN;}


void LED_Initialize(GPIO_Module* GPIOx, uint16_t pin);
void LED_Toggle(GPIO_Module* GPIOx, uint16_t pin);
void LED_On(GPIO_Module* GPIOx,uint16_t pin);
void LED_Off(GPIO_Module* GPIOx,uint16_t pin);

#ifdef __cplusplus
}
#endif

#endif /* __BSP_LED_H__ */


