/*****************************************************************************
 * Copyright (c) 2019, Nations Technologies Inc.
 *
 * All rights reserved.
 * ****************************************************************************
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer below.
 *
 * Nations' name may not be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * DISCLAIMER: THIS SOFTWARE IS PROVIDED BY NATIONS "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL NATIONS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ****************************************************************************/

/**
*\*\file main.c
*\*\author Nations
*\*\version v1.0.0
*\*\copyright Copyright (c) 2019, Nations Technologies Inc. All rights reserved.
**/
#include "main.h"
#include "log.h"
#include "i2c_drv.h"
#include "oled.h"
#include "bsp_delay.h"
#include "math.h"
#include "dma.h"
#include "adc.h"
#include "bsp_led.h"

float voltage = 0.0;      //USB_Aѹ
float electricity = 0.0;  //USB_A
float power = 0.0;        //USB_A
float v_change = 0.0;     //ɵѹ
extern __IO uint16_t ADCConvertedValue[3];

void RCC_Configuration(void);
void NVIC_Configuration(void);

/**
*\*\name    Main program.
*\*\return  none
**/

int main(void)
{
    /* System clocks configuration ---------------------------------------------*/
    RCC_Configuration();
    
		log_init();
	
		i2c_master_init();
	
	  oled_init();
	
		DMA_init();
	
		ADC_init();
	
		LED_Initialize(LED_GPIO_PORT,LED_GPIO_PIN);
		
    while (1)
    {			
				voltage = ADCConvertedValue[0];
				voltage = voltage*3.3f*10.2f/4096;
			
			  electricity = ADCConvertedValue[1];
				electricity = electricity*2*3.3f/4096;
			
				v_change = ADCConvertedValue[2];
				v_change = v_change*3.3f*10.18f/4096;
			
				power = voltage * electricity ;
			
			
			  //OLED
				//ô˺ֹ˸	
//				OLED_ShowString(0,1,(const u8 *)" ",12);
				
				//õѹϢʾOled
				OLED_ShowString(0,1,(const u8 *)"volta",12);
				OLED_ShowFloat(22,1,voltage,2,12);
			  OLED_ShowString(59,1,(const u8 *)"V",12);
			
				OLED_ShowString(0,11,(const u8 *)"elect",12);
				OLED_ShowFloat(22,11,electricity,2,12);
			  OLED_ShowString(59,11,(const u8 *)"A",12);
				
				OLED_ShowString(0,21,(const u8 *)"power",12);
				OLED_ShowFloat(22,21,power,2,12);
				OLED_ShowString(59,21,(const u8 *)"W",12);
				
        OLED_ShowString(0,40,(const u8 *)"v_cha",12);				
				OLED_ShowFloat(22,40,v_change,2,12);
				OLED_ShowString(59,40,(const u8 *)"V",12);	
				//OLDEݲˢ
				OLED_Refresh_Gram();	
			  
					
			  //LED
//				LED_Off(LED_GPIO_PORT,LED_GPIO_PIN);
//				SysTick_Delay_Ms(1000);
//				LED_On(LED_GPIO_PORT,LED_GPIO_PIN);
//				SysTick_Delay_Ms(1000);
//				LED_Toggle(LED_GPIO_PORT,LED_GPIO_PIN);
//				SysTick_Delay_Ms(1000);
				LED_Toggle(LED_GPIO_PORT,LED_GPIO_PIN);
				SysTick_Delay_Ms(1000);
//				LED_OFF;
//				SysTick_Delay_Ms(1000);
//				LED_ON;
//				SysTick_Delay_Ms(1000);
			
			
//			//
//				printf("ڵ\r\n");
//				Usart_SendString(USART1,"chuankoutiaoshi\r\n");
//				printf("%d\r\n",ADCConvertedValue);
    }
}

/**
*\*\name    RCC_Configuration.
*\*\fun     Configures the different system clocks.
*\*\return  none
**/
void RCC_Configuration(void)
{
    /* Enable peripheral clocks ------------------------------------------------*/
    RCC_APB1_Peripheral_Clock_Enable(RCC_APB1_PERIPH_I2C1);
    RCC_APB2_Peripheral_Clock_Enable(RCC_APB2_PERIPH_AFIO | RCC_APB2_PERIPH_USART1);
    RCC_AHB_Peripheral_Clock_Enable(RCC_AHB_PERIPH_DMA|RCC_AHB_PERIPH_GPIOA|RCC_AHB_PERIPH_GPIOB|RCC_AHB_PERIPH_ADC);
    /* RCC_ADCHCLK_DIV16*/
    ADC_Clock_Mode_Config(ADC_CKMOD_AHB, RCC_ADCHCLK_DIV16);
    RCC_ADC_1M_Clock_Config(RCC_ADC1MCLK_SRC_HSE, RCC_ADC1MCLK_DIV8);  //selsect HSE as RCC ADC1M CLK Source
}

#ifdef USE_FULL_ASSERT

/**
*\*\name    assert_failed.
*\*\fun     Reports the name of the source file and the source line number
*\*\        where the assert_param error has occurred.
*\*\param   file pointer to the source file name
*\*\param   line assert_param error line source number
*\*\return  none
**/
void assert_failed(const uint8_t* expr, const uint8_t* file, uint32_t line)
{
    /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

    /* Infinite loop */
    while (1)
    {
    }
}

#endif

